//---------------------------------------------------------------------------
// TestMicroEstablishment
//---------------------------------------------------------------------------
#if !defined(TestMicroEstablishment_H)
  #define TestMicroEstablishment_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the clMicroEstablishment class.
*/
class clTestMicroEstablishment : public CppUnit::TestFixture {

  public:

  /**
  * Collection of other tests.
  */
  void TestMicroEstablishment();

  protected:

  /**
  * Tests error processing.  WriteXMLErrorFile1 is written to see if the
  * expected errors are triggered.
  * @param p_oSimManager Sim manager object.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

 /**Tests the functions clMicroEstablishment::SetFreshLogZAndRespite,
  * SetMoundZAndRespite, and SetGroundZAndRespite.  It passes in a seedling to
  * each to see if the expected results are correct for the last two (greater
  * than 0 for z and 0 for respite in the first case, 0 for both z and respite
  * in the second case).  Then 1000 seeds of species 2 are added to cell (0,0)
  * and the substrate set to all fresh logs.  The fresh log proportions array
  * will be split evenly between the four possible ages: 0, 2, 4, and 6.  Then
  * the seedlings are checked to make sure they're roughly equal to those ages.
  * @param p_oSimManager Sim manager object.
  */
  void TestSetZAndRespite(clSimManager *p_oSimManager);

  /**
  * Produces 1000 seeds in cell 0,0, and lets them all germinate (by setting
  * substrate favorabilities to 1).  WriteXMLFile3() gives us the setup file,
  * with mound proportion = 1.  The substrate type is set to all moss (so fresh
  * logs don't interfere).  Then the z's of the seedlings will be collected and
  * written to MoundZ.txt.
  * @param p_oSimManager Sim manager object.
  */
  void TestMoundZDistributions(clSimManager *p_oSimManager);

  /**
  * Tests the function clMicroEstablishment::SetFreshLogZAndRespite, to make
  * sure that seedling z heights are properly distributed.  This places 1000
  * cells in cell 0,0, and lets them all germinate.  The substrate type is set
  * to all fresh log.  WriteXMLFile3() is used to set up.  Then the z's of the
  * seedlings are collected and written to FreshLogZ.txt.
  */
  void TestFreshLogZDistributions(clSimManager *p_oSimManager);

  /**
  * Tests the function clMicroEstablishment::GetSubstrateProportions.  There
  * are three setup files:  WriteXMLFile2(), WriteXMLFile3(), and
  * WriteXMLFile5(). For all three, specific substrate proportions are assigned
  * to cells and the results compared.  Then we check seed distribution by
  * checking the number of seedlings out of an original 100 that have z values
  * greater than 0.
  * @param p_oSimManager Sim manager object.
  */
  void TestGetSubstrateProportions(clSimManager *p_oSimManager);

  /**
  * Tests the function clMicroEstablishment::GetFreshLogProportions.   There
  * are three setup files:  WriteXMLFile2(), WriteXMLFile3(), and
  * WriteXMLFile5().  For all three, specific substrate proportions are assigned
  * to cells and the results compared.
  * @param p_oSimManager Sim manager object.
  */
  void TestGetFreshLogProportions(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file that sets up 6 species, 3 of which get
  * establishment.
  * @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file that sets up 1 species.  All favorabilities are set
  * to 1.  Mound proportion is set to 1.  Mound mean height is 3 m.  Mound
  * standard deviation is 0.5 m.
  * @return Filename written.
  */
  char * WriteXMLFile2();

  /**
  * Writes a parameter file that sets up 6 species, 3 of which get
  * establishment.  All favorabilities are set to 1.  Mound proportion is set
  * to 0.
  * @return Filename written.
  */
  char * WriteXMLFile3();

  /**
  * Writes a parameter file that contains a mound proportion that is
  * negative.
  * @return Filename written.
  */
  char * WriteXMLErrorFile1();

  /**
  * Writes common portions of the test parameter files (tree pop, plot, etc).
  * This ends with &lt;/allometry&gt.
  * @param oOut File stream to write to.
  */
  void WriteCommonStuff(std::fstream &oOut);
};
//---------------------------------------------------------------------------
#endif // TestMicroEstablishment_H
