//---------------------------------------------------------------------------
// TestPuertoRicoEstablishment
//---------------------------------------------------------------------------
#if !defined(TestPuertoRicoEstablishment_H)
  #define TestPuertoRicoEstablishment_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>

class clSimManager;

/**
* Tests the clPuertoRicoEstablishment class.
*/
class clTestPuertoRicoEstablishment : public CppUnit::TestFixture {

  public:

  /**
  * Tests the clPuertoRicoEstablishment class.
  */
  void TestPuertoRicoEstablishment();

  protected:

  /**
  * This tests normal processing.  A plot is set up and a set of trees added
  * designed to give target GLI values to certain "Dispersed Seed" grid cells.
  * Mostly there is only one neighbor, and the trees are placed such that the
  * GLI point is completely within their canopies, to make test calculations
  * easy.  (GLI calculations themselves don't need testing here.)  A given
  * number of seeds of species 2, 3, and 4 is added to each of 9 grid cells.
  * The run goes, and then we check that the appropriate number of seedlings
  * of each species in each cell exists.  WriteXMLFile1() writes the parameter
  * file.
  * @param p_oSimManager Sim Manager object.
  */
  void TestNormalProcessing(clSimManager *p_oSimManager);

  /**
  * Tests error processing.  It checks for the following errors:
  * <ul>
  * <li>Any of the light extinction coefficient values are not between 0 and 1.</li>
  * <li>The value for m_fLightHeight is not zero or greater</li>
  * <li>The value for mp_fOptimumGLI is not between 0 and 100.</li>
  * <li>There is no disperse behavior enabled.</li>
  * </ul>
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes a setup file.  This contains 4 species, one unused
  * (species 1; this helps catch array-index problems).  Non-spatial disperse
  * is applied to adults of all species, with values such that no seeds will
  * be dispersed from any trees.  Species 2-4 get Puerto Rico establishment.
  *
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes a file where a light extinction coefficient is not between 0 and 1.
  * @return Filename written.
  */
  char* WriteXMLErrorFile1();

  /**
  * Writes a file where the value for m_fLightHeight is not zero or greater.
  * @return Filename written.
  */
  char* WriteXMLErrorFile2();

  /**
  * Writes a file where the value for mp_fOptimumGLI is not between 0 and 100.
  * @return Filename written.
  */
  char* WriteXMLErrorFile3();

  /**
  * Writes a file where there is no disperse behavior.
  * @return Filename written.
  */
  char* WriteXMLErrorFile4();

  /**
  * Writes a file where there is no storm behavior.
  * @return Filename written.
  */
  char* WriteXMLErrorFile5();

  /**
  * Writes a file where the number of altitude angles is less than 1.
  * @return Filename written.
  */
  char* WriteXMLErrorFile6();

  /**
  * Writes a file where the number of azimuth angles is less than 1.
  * @return Filename written.
  */
  char* WriteXMLErrorFile7();


  /**
  * Writes common portions of the test parameter files (tree pop, plot, etc).
  * @param oOut File stream to write to.
  */
  void WriteCommonStuff(std::fstream &oOut);


};
//---------------------------------------------------------------------------
#endif // TestPuertoRicoEstablishment_H
